// $Id: Doxymodules_exoticphysics.h 86514 2014-11-13 09:10:24Z gcosmo $
// The example class categories definitions for Doxygen

/// \file Doxymodules_exoticphysics.h
/// \brief The page that defines the extended/exoticphysics examples modules 


/** @defgroup extended_exoticphysics exoticphysics
 *  Extended examples eventgenerator classes
 *  @{
 */

/** @defgroup extended_exoticphysics_channeling channeling
 *  Exotic physics example channeling
 *  @ingroup extended_exoticphysics
 *  @{
 */

  class ExExChAnalysis {};
  class ExExChAntiBarionBuilder {};
  class ExExChDetectorConstruction {};
  class ExExChDetectorConstructionMessenger {};
  class ExExChEventAction {};
  class ExExChHadronElasticPhysics {};
  class ExExChHadronPhysicsQGSP_BIC {};
  class ExExChHyperonFTFPBuilder {};
  class ExExChIonPhysics {};
  class ExExChParticleUserInfo {};
  class ExExChPhysicsList {};
  class ExExChPhysicsListMessenger {};
  class ExExChPhysListEmStandardSS {};
  class ExExChPiKBuilder {};
  class ExExChPrimaryGeneratorAction {};
  class ExExChProcessChanneling {};
  class ExExChProtonBuilder {};
  class ExExChRunAction {};
  class ExExChSensitiveDetector {};
  class ExExChSensitiveDetectorHit {};
  class ExExChStackingAction {};
  class ExExChStackingActionMessenger {};
  class ExExChTrackingAction {};
  class ExExChUserActionInitialization {};
  class XCrystalCharacteristicArray {};
  class XCrystalIntegratedDensityHub {};
  class XCrystalIntegratedDensityPlanar {};
  class XCrystalPlanarMoliereElectricField {};
  class XCrystalPlanarMoliereElectronDensity {};
  class XCrystalPlanarMolierePotential {};
  class XCrystalPlanarMoliereTempPotential {};
  class XCrystalPlanarNucleiDensity {};
  class XLatticeManager3 {};
  class XLogicalAtomicLatticeDiamond {};
  class XLogicalAtomicLattice {};
  class XLogicalAtomicLatticeSingleAtom {};
  class XLogicalBase {};
  class XLogicalLattice {};
  class XPhysicalLattice {};
  class XUnitCell {};
  class XVCrystalCharacteristic {};
  class XVCrystalIntegratedDensity {};
  class XVCrystalPlanarAnalytical {};
  class XWrapperContinuousDiscreteProcess {};
  class XWrapperDiscreteProcess {};

/** @} */
/** @defgroup extended_exoticphysics_monopole monopole
 *  Exoticphysics monopole example has its
 *  <a href="../html_monopole/html/index.html">standalone documentation </a>
 *  @ingroup extended_exoticphysics
 *  @{
 */

/** @} */

/** @defgroup extended_exoticphysics_ucn ucn
 *  Exotic physics example ucn
 *  @ingroup extended_exoticphysics
 *  @{
 */

  class ExUCNActionInitialization {};
  class ExUCNDetectorConstruction {};
  class ExUCNExtraPhysics {};
  class ExUCNPhysicsList {};
  class ExUCNPrimaryGeneratorAction {};
  class ExUCNRunAction {};

/** @} */

/** @defgroup extended_exoticphysics_phonon phonon
 *  Exotic physics example phonon
 *  @ingroup extended_exoticphysics
 *  @{
 */

  class XActionInitialization {};
  class XAluminumElectrodeHit {};
  class XAluminumElectrodeSensitivity {};
  class XDetectorConstruction {};
  class XPhononStackingAction {};
  class XPhysicsList {};
  class XPrimaryGeneratorAction {};

/** @} */

/** @} */
